==> javie-0.3.0-src.tar.gz: javie-0.3.0-src/ch.kuramo.javie.app/src/main/java/ch/kuramo/javie/app/ApplicationWorkbenchWindowAdvisor.java <==
/*
 * Copyright (c) 2009 Yoshikazu Kuramochi
 * All rights reserved.
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

package ch.kuramo.javie.app;

import java.io.File;
import java.io.FileNotFoundException;
import java.util.Set;

import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.IMemento;
import org.eclipse.ui.IViewReference;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.application.ActionBarAdvisor;
import org.eclipse.ui.application.IActionBarConfigurer;
import org.eclipse.ui.application.IWorkbenchWindowConfigurer;
import org.eclipse.ui.application.WorkbenchWindowAdvisor;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import ch.kuramo.javie.app.project.ProjectEvent;
import ch.kuramo.javie.app.project.ProjectEventHub;
import ch.kuramo.javie.app.project.ProjectListener;
import ch.kuramo.javie.app.project.ProjectManager;
import ch.kuramo.javie.app.views.LayerCompositionView;
import ch.kuramo.javie.app.views.MediaPlayerView;
import ch.kuramo.javie.core.CompositionItem;
import ch.kuramo.javie.core.Item;
import ch.kuramo.javie.core.MediaItem;

public class ApplicationWorkbenchWindowAdvisor
		extends WorkbenchWindowAdvisor implements ProjectListener {

	private static final Logger logger = LoggerFactory.getLogger(ApplicationWorkbenchWindowAdvisor.class);

	private static final String MEMENTO_PROJECT_FILE = "ch.kuramo.javie.app.memento.projectFile";

	private static final String DONE_SAVE_CONFIRMATION = "DONE_SAVE_CONFIRMATION";


	private IMemento _memento;


	public ApplicationWorkbenchWindowAdvisor(IWorkbenchWindowConfigurer configurer) {
		super(configurer);
	}

	public ActionBarAdvisor createActionBarAdvisor(IActionBarConfigurer configurer) {
		return new ApplicationActionBarAdvisor(configurer);
	}

	public IStatus restoreState(IMemento memento) {
		_memento = memento;
		return Status.OK_STATUS;
	}

	public IStatus saveState(IMemento memento) {
		ProjectManager pm = ProjectManager.forWorkbenchWindow(getWindowConfigurer().getWindow());
		if (pm != null && pm.getFile() != null) {
			memento.putString(MEMENTO_PROJECT_FILE, pm.getFile().getAbsolutePath());
		}
		return Status.OK_STATUS;
	}

	public void preWindowOpen() {
		IWorkbenchWindowConfigurer configurer = getWindowConfigurer();
		configurer.setInitialSize(new Point(800, 600));
		configurer.setShowMenuBar(true);
		configurer.setShowCoolBar(false);
		configurer.setShowStatusLine(false);
		configurer.setTitle("Javie");
	}

	public void postWindowCreate() {
		ProjectEventHub.getHub().addProjectListener(this);

		Shell shell = getWindowConfigurer().getWindow().getShell();

		if (_memento == null) {
			shell.setMaximized(true);
		}

		checkViewReferences();

		shell.getDisplay().asyncExec(new Runnable() {
			public void run() {
				openProject();
			}
		});
	}

	private void checkViewReferences() {
		IWorkbenchPage page = getWindowConfigurer().getWindow().getActivePage();
		for (IViewReference viewRef : page.getViewReferences()) {
			String id = viewRef.getId();
			if (id.equals(LayerCompositionView.ID) || id.equals(MediaPlayerView.ID)) {
				viewRef.getView(true);
			}
		}
	}

	private void openProject() {
		ProjectManager pm = null;
		IWorkbenchWindow window = getWindowConfigurer().getWindow();

		String path = (_memento != null) ? _memento.getString(MEMENTO_PROJECT_FILE) : null;
		if (path != null) {
			try {
				pm = ProjectManager.openProject(new File(path), window);
			} catch (FileNotFoundException e) {
				// ignore
			} catch (Exception e) {
				logger.error("error opening project", e);
			}
		}
		if (pm == null) {
			pm = ProjectManager.newProject(window);
		}
	}

	static boolean saveConfirmationRequired(IWorkbenchWindow window) {
		Object data = window.getShell().getData(DONE_SAVE_CONFIRMATION);
		return !Boolean.TRUE.equals(data);
	}

	public boolean preWindowShellClose() {
		IWorkbenchWindow window = getWindowConfigurer().getWindow();
		ProjectManager pm = ProjectManager.forWorkbenchWindow(window);
		if (pm != null) {
			switch (SaveUtil.saveIfDirty(pm)) {
				case NOT_DIRTY:
				case SAVED:
				case UNSAVED:
					window.getShell().setData(DONE_SAVE_CONFIRMATION, Boolean.TRUE);
					return true;
			}
		}
		return false;
	}

	public void postWindowClose() {
		ProjectManager pm = ProjectManager.forWorkbenchWindow(getWindowConfigurer().getWindow());
		if (pm != null) {
			pm.dispose();
		}

		ProjectEventHub.getHub().removeProjectListener(this);
	}

	public boolean isDurableFolder(String perspectiveId, String folderId) {
		return perspectiveId.equals(Perspective.ID) &&
				(folderId.equals(Perspective.ID_CENTER_FOLDER) || folderId.equals(Perspective.ID_BOTTOM_FOLDER));
	}

	private void updateWindowTitle(ProjectManager pm) {
		File file = pm.getFile();
		boolean dirty = pm.isDirty();

		String title = (file != null) ? file.getName() : ProjectManager.DEFAULT_NAME;
		if (dirty) {
			title += " *";
		}

		Shell shell = getWindowConfigurer().getWindow().getShell();
		shell.setText(title);
		shell.setModified(dirty);
	}

	private void resetWindowTitle() {
		Shell shell = getWindowConfigurer().getWindow().getShell();
		shell.setText("Javie");
		shell.setModified(false);
	}

	private void hideViews(Set<Item> items) {
		IWorkbenchPage page = getWindowConfigurer().getWindow().getActivePage();

		for (Item item : items) {
			if (item instanceof MediaItem) {
				IViewReference viewRef = page.findViewReference(MediaPlayerView.ID, item.getId());
				if (viewRef != null) {
					page.hideView(viewRef);
				}
			}
			if (item instanceof CompositionItem) {
				IViewReference viewRef = page.findViewReference(LayerCompositionView.ID, item.getId());
				if (viewRef != null) {
					page.hideView(viewRef);
				}
			}
		}
	}

	public void handleProjectEvent(ProjectEvent event) {
		ProjectManager pm = event.getProjectManager();
		if (pm.getWorkbenchWindow() != getWindowConfigurer().getWindow()) {
			return;
		}

		switch (event.type) {
			case PROJECT_INITIALIZE:
			case DIRTY_CHANGE:
			case FILE_CHANGE:
				updateWindowTitle(pm);
				break;

			case PROJECT_DISPOSE:
				resetWindowTitle();
				break;

			case ITEMS_REMOVE:
				hideViews(ProjectEvent.getItems(event));
				break;
		}
	}

}

==> javie-0.3.4-src.tar.gz: javie-0.3.4-src/ch.kuramo.javie.app/src/main/java/ch/kuramo/javie/app/ApplicationWorkbenchWindowAdvisor.java <==
/*
 * Copyright (c) 2009,2010 Yoshikazu Kuramochi
 * All rights reserved.
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

package ch.kuramo.javie.app;

import java.io.File;
import java.util.Set;

import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.IMemento;
import org.eclipse.ui.IViewReference;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.application.ActionBarAdvisor;
import org.eclipse.ui.application.IActionBarConfigurer;
import org.eclipse.ui.application.IWorkbenchWindowConfigurer;
import org.eclipse.ui.application.WorkbenchWindowAdvisor;

import ch.kuramo.javie.app.actions.NewProjectAction;
import ch.kuramo.javie.app.project.ProjectEvent;
import ch.kuramo.javie.app.project.ProjectEventHub;
import ch.kuramo.javie.app.project.ProjectListener;
import ch.kuramo.javie.app.project.ProjectManager;
import ch.kuramo.javie.app.views.LayerCompositionView;
import ch.kuramo.javie.app.views.MediaPlayerView;
import ch.kuramo.javie.core.CompositionItem;
import ch.kuramo.javie.core.Item;
import ch.kuramo.javie.core.MediaItem;

public class ApplicationWorkbenchWindowAdvisor
		extends WorkbenchWindowAdvisor implements ProjectListener {

	private static final String DONE_SAVE_CONFIRMATION = "DONE_SAVE_CONFIRMATION";


	private boolean _maximizeShell = true;


	public ApplicationWorkbenchWindowAdvisor(IWorkbenchWindowConfigurer configurer) {
		super(configurer);
	}

	public ActionBarAdvisor createActionBarAdvisor(IActionBarConfigurer configurer) {
		return new ApplicationActionBarAdvisor(configurer);
	}

	public IStatus restoreState(IMemento memento) {
		_maximizeShell = false;
		return Status.OK_STATUS;
	}

	public void preWindowOpen() {
		IWorkbenchWindowConfigurer configurer = getWindowConfigurer();
		configurer.setInitialSize(new Point(800, 600));
		configurer.setShowMenuBar(true);
		configurer.setShowCoolBar(false);
		configurer.setShowStatusLine(false);
		configurer.setTitle("Javie");
	}

	public void postWindowCreate() {
		ProjectEventHub.getHub().addProjectListener(this);

		Shell shell = getWindowConfigurer().getWindow().getShell();

		if (_maximizeShell) {
			shell.setMaximized(true);
		}

		shell.getDisplay().asyncExec(new Runnable() {
			public void run() {
				NewProjectAction.newProject(getWindowConfigurer().getWindow());
			}
		});
	}

	static boolean saveConfirmationRequired(IWorkbenchWindow window) {
		Object data = window.getShell().getData(DONE_SAVE_CONFIRMATION);
		return !Boolean.TRUE.equals(data);
	}

	public boolean preWindowShellClose() {
		IWorkbenchWindow window = getWindowConfigurer().getWindow();
		ProjectManager pm = ProjectManager.forWorkbenchWindow(window);
		if (pm != null) {
			switch (SaveUtil.saveIfDirty(pm)) {
				case NOT_DIRTY:
				case SAVED:
				case UNSAVED:
					window.getShell().setData(DONE_SAVE_CONFIRMATION, Boolean.TRUE);
					return true;
			}
		}
		return false;
	}

	public void postWindowClose() {
		ProjectManager pm = ProjectManager.forWorkbenchWindow(getWindowConfigurer().getWindow());
		if (pm != null) {
			pm.dispose();
		}

		ProjectEventHub.getHub().removeProjectListener(this);
	}

	private void updateWindowTitle(ProjectManager pm) {
		File file = pm.getFile();
		boolean dirty = pm.isDirty();

		String title = (file != null) ? file.getName() : ProjectManager.DEFAULT_NAME;
		if (dirty) {
			title += " *";
		}

		Shell shell = getWindowConfigurer().getWindow().getShell();
		shell.setText(title);
		shell.setModified(dirty);
	}

	private void resetWindowTitle() {
		Shell shell = getWindowConfigurer().getWindow().getShell();
		shell.setText("Javie");
		shell.setModified(false);
	}

	private void hideViews(Set<Item> items) {
		IWorkbenchPage page = getWindowConfigurer().getWindow().getActivePage();

		for (Item item : items) {
			if (item instanceof MediaItem) {
				IViewReference viewRef = page.findViewReference(MediaPlayerView.ID, item.getId());
				if (viewRef != null) {
					page.hideView(viewRef);
				}
			}
			if (item instanceof CompositionItem) {
				IViewReference viewRef = page.findViewReference(LayerCompositionView.ID, item.getId());
				if (viewRef != null) {
					page.hideView(viewRef);
				}
			}
		}
	}

	public void handleProjectEvent(ProjectEvent event) {
		ProjectManager pm = event.getProjectManager();
		if (pm.getWorkbenchWindow() != getWindowConfigurer().getWindow()) {
			return;
		}

		switch (event.type) {
			case PROJECT_INITIALIZE:
			case DIRTY_CHANGE:
			case FILE_CHANGE:
				updateWindowTitle(pm);
				break;

			case PROJECT_DISPOSE:
				resetWindowTitle();
				break;

			case ITEMS_REMOVE:
				hideViews(ProjectEvent.getItems(event));
				break;
		}
	}

}
